#!/usr/bin/env node
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/**
 * This script checks the current Pull Request for any previous deployment comments,
 * deletes them, and then adds a comment with a link to the new deployment URL.
 */


var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var commentPoster = function commentPoster(_ref) {
  var _ref$prUrl = _ref.prUrl,
      prUrl = _ref$prUrl === undefined ? null : _ref$prUrl,
      _ref$ghAuthTokenUsern = _ref.ghAuthTokenUsername,
      ghAuthTokenUsername = _ref$ghAuthTokenUsern === undefined ? null : _ref$ghAuthTokenUsern,
      _ref$ghAuthToken = _ref.ghAuthToken,
      ghAuthToken = _ref$ghAuthToken === undefined ? null : _ref$ghAuthToken,
      _ref$repoUsername = _ref.repoUsername,
      repoUsername = _ref$repoUsername === undefined ? null : _ref$repoUsername,
      _ref$repoName = _ref.repoName,
      repoName = _ref$repoName === undefined ? null : _ref$repoName,
      _ref$deploymentUrl = _ref.deploymentUrl,
      deploymentUrl = _ref$deploymentUrl === undefined ? null : _ref$deploymentUrl,
      _ref$customMessage = _ref.customMessage,
      customMessage = _ref$customMessage === undefined ? 'Beep boop. Your code has been deployed!' : _ref$customMessage;

  // eslint-disable-next-line max-len
  if (!prUrl || !ghAuthTokenUsername || !ghAuthToken || !repoUsername || !repoName || !deploymentUrl) {
    return Promise.reject(new Error('All required input parameters for cleanup were not provided.'));
  }

  var githubHeaders = {
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
      'X-GitHub-Media-Type': 'github.v3',
      Authorization: 'Basic ' + Buffer.from(ghAuthTokenUsername + ':' + ghAuthToken).toString('base64')
    }
  };

  var regex = new RegExp(customMessage + ' https://.+.now.sh/?', 'g');
  var prNumber = prUrl.replace('https://github.com/' + repoUsername + '/' + repoName + '/pull/', '');

  return (0, _isomorphicFetch2.default)('https://api.github.com/repos/' + repoUsername + '/' + repoName + '/issues/' + prNumber + '/comments', githubHeaders).then(function (res) {
    return res.json();
  })
  // Fetch all comments from this Pull Request.
  .then(function (response) {
    return Promise.all(response.map(function (comment) {
      return (0, _isomorphicFetch2.default)(comment.url, githubHeaders).then(function (res) {
        return res.json();
      });
    }));
  })
  // Find comments which match the pattern for posting build links,
  // and then send a DELETE request.
  .then(function (comments) {
    return Promise.all(comments.filter(function (comment) {
      var match = regex.exec(comment.body);
      return match && match.length > 0;
    }).map(function (comment) {
      return (0, _isomorphicFetch2.default)(comment.url, Object.assign({}, githubHeaders, { method: 'DELETE' }));
    }));
  })
  // Add a new comment with the new build URL.
  .then(function () {
    return (0, _isomorphicFetch2.default)('https://api.github.com/repos/' + repoUsername + '/' + repoName + '/issues/' + prNumber + '/comments', _extends({}, githubHeaders, {
      method: 'POST',
      body: JSON.stringify({
        body: customMessage + ' ' + deploymentUrl
      })
    }));
  });
};

exports.default = commentPoster;